/**
 * Desenvolvido por Mdulo Security Solutions S.A.
 * Contato: Eduardo Nery (enery@modulo.com.br - 61.218.7500)
 * Site: http://www.modulo.com.br 
 **/

#include "kgreet_nss.h"

#include <klocale.h>
#include <klineedit.h>
#include <kpassdlg.h>
#include <kuser.h>

#include <qregexp.h>
#include <qlayout.h>
#include <qlabel.h>
#include <ktextedit.h>
#include <qmessagebox.h>

#ifdef DEBUG_GREET
  #define D(x) QMessageBox::information(NULL, "", x)
#else
  #define D(x)
#endif

static int echoMode;

KNSSGreeter::KNSSGreeter(KGreeterPluginHandler *_handler, QWidget *parent,
                         QWidget *predecessor, const QString &_fixedEntity, Function _func, Context _ctx )
    : QObject(), KGreeterPlugin( _handler ), m_fixedUser( _fixedEntity ), m_func( _func ), m_ctx( _ctx ), m_state(StateLogin), m_aborted(false)
{
  D(QString("FixedEntity: %1, Function: %2, Ctx: %3").arg(
      m_fixedUser, QString::number(m_func), QString::number(m_ctx)));


  m_box = new QHBoxLayout( parent );
  layoutItem = m_box;

  m_label = new QLabel(parent);//, i18n("&Username:"), parent );

  m_edLogin = new KLineEdit( parent );
  m_edLogin->setContextMenuEnabled(false);
  m_edLogin->setText(m_fixedUser);  
  parent->setTabOrder(predecessor, m_edLogin);
  m_edLogin->hide();

  m_edPKPassword = new KPasswordEdit( parent );
  if (echoMode == -1) {
    m_edPKPassword->setEchoMode( static_cast<QLineEdit::EchoMode>(echoMode));
  }

  m_edPKPassword->hide();

  m_box->addWidget(m_label);
  m_box->addWidget(m_edLogin);
  m_box->addWidget(m_edPKPassword);
}

void KNSSGreeter::setPrompt(const char *prompt, bool echo)
{
  m_label->setText(i18n(prompt));
  if(echo) {
    m_state = StateLogin;
    m_edPKPassword->hide();
    m_edLogin->setEnabled(true);    
    m_edLogin->show();
    m_edLogin->setFocus();
  } else {	  
    m_state = StatePassword;
    m_edLogin->hide();
    m_edPKPassword->setEnabled(true);
    m_edPKPassword->erase();
    m_edPKPassword->show();
    m_edPKPassword->setFocus();
  }
}

KNSSGreeter::~KNSSGreeter()
{
  D("dtor");  
  //abort();
  QLayoutIterator it = static_cast<QLayout *>(layoutItem)->iterator();
  for (QLayoutItem *itm = it.current(); itm; itm = ++it) {
    delete itm->widget();
  }
  delete layoutItem;
}

void KNSSGreeter::loadUsers( const QStringList &users )
{
  D(QString("KNSSGreeter::loadUsers, users: %1").arg(users.join(":")));

  KCompletion *userNamesCompletion = new KCompletion;
  userNamesCompletion->setItems( users );
  m_edLogin->setCompletionObject( userNamesCompletion );
  m_edLogin->setAutoDeleteCompletionObject( true );
  m_edLogin->setCompletionMode( KGlobalSettings::CompletionAuto );
}

void KNSSGreeter::presetEntity( const QString &entity, int field )
{
  D(QString("KNSSGreeter::presetEntity, entity: %1, field: %2").arg(
      entity, QString::number(field)));

  switch(field) {
  case FieldLogin:
    m_edLogin->setText(entity);
    break;
  case FieldKeyPass:
    break;
  }
  handler->gplugSetUser(entity);
}

QString KNSSGreeter::getEntity() const
{
  D(QString("KNSSGreeter::getEntity"));
  return m_edLogin->text();
}

void KNSSGreeter::setUser( const QString &user )
{
  D(QString("KNSSGreeter::setUser: " + user));
  m_edLogin->setText(user);
}

void KNSSGreeter::setEnabled( bool on )
{
  D(QString("KNSSGreeter::setEnabled, on(true|false): ") + QString::number(on));
}

bool KNSSGreeter::textMessage( const char *message, bool error )
{
  D(QString("KNSSGreeter::textMessage, message: %1, error(t|f): %2").arg(
      QString(message), QString::number(error)));
}

void KNSSGreeter::textPrompt(const char *prompt, bool echo, bool nonBlocking)
{
  D(QString("KNSSGreeter::textPrompt, prompt: %1, echo %2, nonBlocking: %3").arg(
      QString(prompt), QString::number(echo), QString::number(nonBlocking)));

  setPrompt(prompt, echo);
  
  if(nonBlocking) {
    //shouldn`t happen
    return;
  }
/*    
    D(QString("KNSSGreeter::textPrompt() ops, nonBlocking == true"));
    //shouldn'y happend

    if(m_state == StateLogin) {
      handler->gplugReturnText(
        m_edLogin->text().local8Bit(),
          KGreeterPluginHandler::IsUser );
    } else {
      handler->gplugReturnText(
        m_edPKPassword->password(),
          KGreeterPluginHandler::IsPassword );
    }
  }*/
}


bool KNSSGreeter::binaryPrompt( const char *prompt, bool nonBlocking )
{
    //not used
    D(QString("KNSSGreeter::binaryPromopt"));
    return true;
}

void KNSSGreeter::start()
{
  D(QString("KNSSGreeter::start"));

  //prevent from crashing
  if(!m_aborted)
     handler->gplugStart();
}

void KNSSGreeter::suspend()
{
  //haven`t used..
  D(QString("KNSSGreeter::suspend"));
}

void KNSSGreeter::resume()
{
  //haven`t used
  D(QString("KNSSGreeter::resume"));
}


void KNSSGreeter::next()
{
  D(QString("KNSSGreeter::next"));
  
  if(m_state == StateLogin) {
      //m_state = StatePassword;
      
      m_edLogin->setEnabled(false);      

      D("setting handler->user to what is on edLogin");

      handler->gplugSetUser(m_edLogin->text());
      
      D(QString("KNSSGreeter::next() returning user: ") + m_edLogin->text());
      handler->gplugReturnText(
          m_edLogin->text().local8Bit(),
          KGreeterPluginHandler::IsUser );
  } else {
    m_edPKPassword->setEnabled(false);
    
    handler->gplugReturnText(
        m_edPKPassword->password(),
        KGreeterPluginHandler::IsPassword );
  }
}

void KNSSGreeter::abort()
{
  D(QString("KNSSGreeter::abort, gplugReturnText(0,0)"));
  m_aborted = true;
//  handler->gplugReturnText(0,0);
}


void KNSSGreeter::succeeded()
{
  D(QString("KNSSGreeter::succeeded"));
}

void KNSSGreeter::failed()
{
  //m_edLogin->setEnabled(false);
  //m_edPKPassword->setEnabled(false);
  D(QString("KNSSGreeter::failed"));
}


void KNSSGreeter::revive()
{
  m_aborted = false;
  D(QString("KNSSGreeter::revive"));
  m_edLogin->setEnabled(true);
  m_edLogin->clear();
  m_edPKPassword->setEnabled(true);
  m_edPKPassword->erase();
  m_state = StateLogin;   
}


void KNSSGreeter::clear()
{
  D(QString("KNSSGreeter::clear"));
  m_edPKPassword->erase();
  m_edLogin->clear();
 
}


// factory

static bool init(
  const QString &,
  QVariant (*getConf)( void *, const char *, const QVariant & ),
  void *ctx )
{
  echoMode = getConf( ctx, "EchoMode", QVariant( -1 ) ).toInt();
  KGlobal::locale()->insertCatalogue("kgreet_nss");
  return true;
}

static void done( void )
{
  KGlobal::locale()->removeCatalogue("kgreet_nss");
}

static KGreeterPlugin* create(
  KGreeterPluginHandler *handler, QWidget *parent, QWidget *predecessor,
  const QString &fixedEntity,  KGreeterPlugin::Function func, KGreeterPlugin::Context ctx )
{
  return new KNSSGreeter( handler, parent, predecessor, fixedEntity, func, ctx );
}

kgreeterplugin_info kgreeterplugin_info = {
                                            I18N_NOOP("Username + NSS Private key password (nss)"), "nss", kgreeterplugin_info::Local, init, done, create
                                          };

#include "kgreet_nss.moc"
