/**
 * Desenvolvido por Mdulo Security Solutions S.A.
 * Contato: Eduardo Nery (enery@modulo.com.br - 61.218.7500)
 * Site: http://www.modulo.com.br
 **/

 
#include "certutil.h"
#include "log.h"

/*
 * The dbdir is something like /home/me/.mozilla/default/xyz123
 * Since the last folder is somewhat random, we have to guess it
 * by opening and searching for a slt sufix
 */
int build_nss_dbdir(const char* homepath, char** dbdir) {	
  /*char* sufix = "/.mozilla/default/f4z2jg8w.slt";*/
	
	char* mozilladir = "/.mozilla/default";
	char* mozpath = NULL;
	char* randomdir = NULL;
	int ret = SECFailure;
	
	DIR *dp = NULL;
        struct dirent *ep = NULL;
	
	mozpath = (char*) malloc(sizeof(char)*(strlen(homepath)+strlen(mozilladir)+1));

	if(!mozpath) {
		ret = SECFailure;
		goto end;
	}		
	
	strcpy(mozpath, homepath);
	strcat(mozpath, mozilladir);
	
  dp = opendir (mozpath);
  if (dp != NULL) {
  	while (ep = readdir(dp)) {
			if((strlen(ep->d_name) >=4) &&
			   (strcmp(".slt", &(ep->d_name[strlen(ep->d_name)-4]))==0)) {
			   	randomdir = (char*) malloc(sizeof(char) * strlen(ep->d_name)+1);
				strcpy(randomdir, ep->d_name);
			}
		}
   closedir (dp);
  }
  else {
		ret = SECFailure;
		goto end;
	}    

  *dbdir = (char*) malloc(sizeof(char) * (strlen(mozpath)+1+strlen(randomdir)+1));
	
	if(!dbdir) {
		ret = SECFailure;
		goto end;
	}		
	
  strcpy(*dbdir, mozpath);
	strcat(*dbdir, "/");
  strcat(*dbdir, randomdir);
	ret = SECSuccess;
	
	end:
		if(mozpath) free(mozpath);
		mozpath = NULL;
		return ret;
}

/*
 * something like /home/me/.eid/authorized_certificates
 */ 
int build_certxtpath(const char* homepath, char** certxtpath) {
  char* sufix = "/.eid/authorized_certificates";
	
  *certxtpath = (char*) malloc(sizeof(char) * (strlen(sufix) + strlen(homepath))+1);
	
	if(!certxtpath) {
		return SECFailure;
	}
	
  strcpy(*certxtpath, homepath);
  strcat(*certxtpath, sufix);
  return SECSuccess;
}
/*
 * Speaks for itself
 */ 
int build_db_file_path(char* db_dir, char **dbfilepath) {

	*dbfilepath	= (char*) 
		malloc(sizeof(char) * (strlen(db_dir)+1+(strlen(g_db_file))+1));
												  /*[dbdir][/][secmod.db]\0*/
	if(!dbfilepath) {
		return SECFailure;
	}
	
	strcpy(*dbfilepath, db_dir);
	strcat(*dbfilepath, "/");
	strcat(*dbfilepath, g_db_file);
	
	return SECSuccess;
}

/*
 * Check wheter we need a password to access
 * the private key from the user's mozilla db
 */ 
int check_for_key_pass(const char* homepath) {
  CERTCertificate  *nss_cert   = NULL;
  char             *certxtpath = NULL;
	char             *dbfilepath = NULL;
  int              ret;

  ret = build_nss_dbdir(homepath, &g_db_dir);

  if(ret == SECFailure) {  goto end;  }
	
	_log_debug("nss_db_dir: %s", g_db_dir);
	
	ret = build_db_file_path(g_db_dir, &dbfilepath);
	
	if(ret == SECFailure) {  goto end;  }
	
	_log_debug("dbfilepath: %s", dbfilepath);
	
	if(access(dbfilepath, F_OK) != 0) { /*db exists?*/
		ret = SECFailure;
		goto end; 
	}
  
  ret = init_nss();
	
  if(ret == SECFailure) {
    _log_debug("init_nss() returned failure");
    ret = SECFailure;
    goto end;
  }

  ret  = build_certxtpath(homepath, &certxtpath);
	
	if(ret == SECFailure) { goto end; }		
	
	_log_debug("certxtdir: %s", certxtpath);
	
  nss_cert   = get_nss_cert(certxtpath);

  if(!nss_cert) {
    _log_debug("unable to get nss_cert");
    ret = SECFailure;
    goto end;
  }

  check_privkey_pass(nss_cert);
	ret = SECSuccess;
	
  end:
    /*TODO: free cert, dbhandler, g_slot, keys, etc */
    if(g_slot) {  PK11_FreeSlot(g_slot); }
		g_slot = NULL;
    if(PR_Initialized() == PR_TRUE) {  NSS_Shutdown(); }
    if(nss_cert) { CERT_DestroyCertificate(nss_cert); }
		nss_cert = NULL;
    if(certxtpath) free(certxtpath); 
		certxtpath = NULL;
    if(g_db_dir) { free(g_db_dir); }
		g_db_dir = NULL;
		if(dbfilepath) free(dbfilepath);
		dbfilepath = NULL;
		_log_debug("process_nss_auth returning: %d", ret);
    return ret;
}

/*
 * Process authentication
 */

int process_nss_auth(const char* homepath, const char* keypass) {
  CERTCertificate  *nss_cert   = NULL;
  char             *certxtpath = NULL;
	char             *dbfilepath = NULL;
  int              ret;
	
	g_keypass = (char*) keypass;

  ret = build_nss_dbdir(homepath, &g_db_dir);

  if(ret == SECFailure) {  goto end;  }
	
	_log_debug("nss_db_dir: %s", g_db_dir);
	
	ret = build_db_file_path(g_db_dir, &dbfilepath);
	
	if(ret == SECFailure) {  goto end;  }
	
	_log_debug("dbfilepath: %s", dbfilepath);
	
	if(access(dbfilepath, F_OK) != 0) { /*db exists?*/
		ret = SECFailure;
		goto end; 
	}
  
  ret = init_nss();
	
  if(ret == SECFailure) {
    _log_debug("init_nss() returned failure");
    ret = SECFailure;
    goto end;
  }

  ret  = build_certxtpath(homepath, &certxtpath);
	
	if(ret == SECFailure) { goto end; }		
	
	_log_debug("certxtdir: %s", certxtpath);
	
  nss_cert   = get_nss_cert(certxtpath);

  if(!nss_cert) {
    _log_debug("unable to get nss_cert");
    ret = SECFailure;
    goto end;
  }

  ret = chanllenge_it(nss_cert);
  if(ret == SECFailure) {
    _log_debug("challenge failed");
    ret = SECFailure;
    goto end;
  } else {
    ret =  SECSuccess;
    goto end;
  }

  end:
    if(g_slot) {  PK11_FreeSlot(g_slot); }
		g_slot = NULL;
    if(PR_Initialized() == PR_TRUE) {  NSS_Shutdown(); }
    if(nss_cert) { CERT_DestroyCertificate(nss_cert); }
		nss_cert = NULL;
    if(certxtpath) free(certxtpath); 
		certxtpath = NULL;
    if(g_db_dir) { free(g_db_dir); }
		g_db_dir = NULL;
		if(dbfilepath) free(dbfilepath);
		dbfilepath = NULL;
		_log_debug("process_nss_auth returning: %d", ret);
    return ret;
}

/* usage: checkcert a|c homepath [keypassword]*/

int main(int argc, char** argv) {
	g_db_dir           = NULL;
	g_slot             = NULL;
	g_keypass          = NULL;
	g_key_request_flag = 0;
	g_db_file = "secmod.db";	
	
	if(argc < 2) {
		goto argerror;
	}		
	
	if(*argv[1] == 'a') {		/* authenticate with privkey pass */		
		int ret;		
		if(argc == 4) {
			_log_debug("processing authentication with passwd");
			ret = process_nss_auth(argv[2], argv[3]);
		} else if(argc == 3) {
			_log_debug("processing authentication w/o passwd");
			ret = process_nss_auth(argv[2], NULL);
		} else {
			goto argerror;
		}
		
		if(SECSuccess == ret) {
			_log_debug("auth success");
			exit(AUTH_SUCCESS);
		} else {
			_log_debug("auth failed");
			exit(AUTH_ERROR);
		}
		
	} else if(*argv[1] == 'c') { /*chec for privkey pass */
		if(argc != 3) {
			goto argerror;
		}	
		
		_log_debug("check_for_key_pass: argv2: %s", argv[2]);
		
		switch(check_for_key_pass(argv[2])) { 
			case SECFailure:
				_log_debug("check failed");
				exit(CHECK_ERROR);
			case SECSuccess:
				if(g_key_request_flag == 0) {
					_log_debug("no pass requested");
					exit(NO_KEY_PASS);
				} else {
					_log_debug("pass requested");
					exit(NEED_KEY_PASS);
				}		  		
		}		
	} 
	
	argerror:
		printf("Invalid argument\n");
		exit(AUTH_ERROR);		
}
