/**
 * Desenvolvido por Mdulo Security Solutions S.A.
 * Contato: Eduardo Nery (enery@modulo.com.br - 61.218.7500)
 * Site: http://www.modulo.com.br
 **/

#define MOD_SUCCESS 0
#define MOD_ERROR   1

#define PAM_SM_AUTH
#include <security/pam_modules.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "common.h"

pam_handle_t *g_pamh;

/*
 * Retrieve pass from user
 */ 
int retrieve_passwd(char** passwd) {

	struct pam_conv *pc     = 0;
	struct pam_message *msg = 0;
	int retval              = MOD_SUCCESS;
		
  retval = pam_get_item(g_pamh, PAM_CONV, (const void**) &pc);
  if(retval != PAM_SUCCESS) {
    retval = MOD_ERROR;					
		goto end;
	}	
	
	msg = (struct pam_message*) malloc(sizeof(struct pam_message));
	if(!msg) {
		retval = MOD_ERROR;
		goto end;
	}

	msg->msg_style = PAM_PROMPT_ECHO_OFF;
	msg->msg = "Private key password: ";

	struct pam_response *resp;
	resp = (struct pam_response*) malloc(sizeof(struct pam_response));
	
	retval = pc->conv(1, (const struct pam_message**) &msg, &resp, pc->appdata_ptr);
	if(retval == PAM_SUCCESS) {
		*passwd = resp->resp;
		retval = MOD_SUCCESS;
	} else {
		*passwd = NULL;
		retval = MOD_ERROR;
	}	
	
	end:
		if(msg)  free(msg);
		if(resp) free(resp);
		return retval;
}

/*
 * build the command to check for private key
 */ 
int build_check_key_cmd(const char* homepath, char** cmd) {
	char* cc = "nssauth ";

	*cmd = (char*) malloc(sizeof(char) * 
		(strlen(cc)	+ 3 + strlen(homepath)+1));
		/*[checkert ][ ][c][ ][homepath][\0] */
	
	if(!*cmd) return MOD_ERROR;
		
	strcpy(*cmd, cc);
	strcat(*cmd, " c ");
	strcat(*cmd, homepath);
	return MOD_SUCCESS;	
}

/*
 * Build the command to check for authentication
 */ 
int build_nssauth_cmd(const char* homepath, const char* passwd, char** cmd) {
	char* cc = "nssauth ";
	if(passwd) {
		*cmd = (char*) malloc(sizeof(char) * 
			(strlen(cc) + 3 + strlen(homepath)+1+strlen(passwd)+1));
		/*[checkert ][ ][a][ ][homepath][ ][passwd][\0] */
  } else {
		*cmd = (char*) malloc(sizeof(char) * 
			(strlen(cc) + 3 + strlen(homepath)+1));
		/*[checkert ][ ][n][ ][homepath][\0] */		
	}
	
	if(!*cmd) return MOD_ERROR;
		
	strcpy(*cmd, cc);
	strcat(*cmd, " a ");
	strcat(*cmd, homepath);
	if(passwd) { strcat(*cmd, " "); strcat(*cmd, passwd); }
	return MOD_SUCCESS;
}

int build_homepath(const char* login, char** homepath) {
  char* base = "/home/";
  
	if(strcmp("root", login) == 0) {
		*homepath = (char*) malloc(sizeof(char) * (1 + strlen(login)+1));
		if(!*homepath) return MOD_ERROR;
		
		strcpy(*homepath, "/");
		strcat(*homepath, login);		
	} else {
  	*homepath = (char*) malloc(sizeof(char) * (strlen(base) + strlen(login)+1));
		if(!*homepath) return MOD_ERROR;
		
    strcpy(*homepath, base);
    strcat(*homepath, login);		
	}  
  
  return MOD_SUCCESS;
}

/*
 * authentication without private key password
 */ 
int auth_without_key(const char* homepath) {
	char* nssauth_cmd = NULL;
	int   retval;
	
	retval = build_nssauth_cmd(homepath, NULL, &nssauth_cmd);
	
	/*printf("M: without| system(%s): %d\n", nssauth_cmd, system(nssauth_cmd));*/
	if(retval == MOD_SUCCESS) {
		retval = system(nssauth_cmd);
	}

  if(nssauth_cmd) free(nssauth_cmd);  
  return retval;
}

/*
 * Guess...
 */ 
int auth_with_key(const char* homepath) {
	char* nssauth_cmd = NULL;
	int   retval;	
	char* passwd = NULL;
	
	/*printf("M: with| retrieving pass\n");*/
	
	retval = retrieve_passwd(&passwd);
	if(retval == MOD_ERROR) {
		goto end;
	}
	
	/*printf("M: password: %s\n", passwd);*/
	
	retval = build_nssauth_cmd(homepath, passwd, &nssauth_cmd);
	
	if(retval == MOD_SUCCESS) {
		/*printf("M: system(%s): %d\n", nssauth_cmd, system(nssauth_cmd));*/
		retval = system(nssauth_cmd);
	}

	end:
  	if(nssauth_cmd) free(nssauth_cmd);  
  	return retval;
}

PAM_EXTERN int pam_sm_authenticate(pam_handle_t *ph, int flags, int argc, const char **argv) {
  int retval = 0;
  const char *login = NULL;
  char* homepath = NULL;
	char* check_key_cmd = NULL;
		
	g_pamh = ph;
  
	/*printf("M: pam_sm_authenticate\n");*/
	
  retval = pam_get_user(g_pamh, &login, "Login: ");
  if(retval != PAM_SUCCESS) {
    goto end;    
  }
	/*printf("M: user: %s\n", login);*/
	
  retval = build_homepath(login, &homepath);
	
  if(retval == MOD_ERROR) {
    retval = PAM_AUTH_ERR;
    goto end;
  }

	retval = build_check_key_cmd(homepath, &check_key_cmd);
	/*printf("M: checkkey cmd: %s\n", check_key_cmd);*/
	
	/*printf("M: system check_key_cmd: %d\n", system(check_key_cmd));*/
	
	switch(system(check_key_cmd) / 0x100) {
	
		case NO_KEY_PASS:
			/*printf("M: no keypass\n");*/
			if(auth_without_key(homepath) == MOD_SUCCESS) {
				retval = PAM_SUCCESS;
			} else {
				retval = PAM_AUTH_ERR;
			}
			break;
		case NEED_KEY_PASS:			
			/*printf("M: need keypass\n");*/
			if(auth_with_key(homepath) == MOD_SUCCESS) {
				retval = PAM_SUCCESS;				
			} else {
				retval = PAM_AUTH_ERR;
			}								
			break;
		case CHECK_ERROR:
		default:
			/*printf("M: check_error\n");*/
    	retval = PAM_AUTH_ERR;    
	}		

  end: 
		if(check_key_cmd) free(check_key_cmd);
    if(homepath) free(homepath);
    return retval;			
}

PAM_EXTERN
int pam_sm_setcred(pam_handle_t *pamh,int flags,int argc
       ,const char **argv)
{
  return PAM_SUCCESS;
}
